@@ -6,6 +6,8 @@
 
 import pytest
 import os
+import h5py
+import itertools
 import numpy as np
 
 from pyuvdata import UVData
@@ -26,6 +28,11 @@ testfiles = [
     "1131733552_mod.metafits",
     "1131733552_mini_cotter.uvfits",
     "1131733552_metafits_ppds.fits",
+    "1061315448_20130823175130_mini_gpubox07_01.fits",
+    "1061315448.metafits",
+    "1061315448_20130823175130_mini_vv_07_01.uvh5",
+    "1320409688.metafits",
+    "1320409688_20211108122750_mini_ch137_000.fits",
 ]
 filelist = [os.path.join(testdir, filei) for filei in testfiles]
 
@@ -73,7 +80,6 @@ def test_read_mwa_write_uvfits(tmp_path):
     mwa_uv = UVData()
     uvfits_uv = UVData()
     messages = [
-        "telescope_location is not set",
         "some coarse channel files were not submitted",
     ]
     with uvtest.check_warnings(UserWarning, messages):
@@ -84,25 +90,78 @@ def test_read_mwa_write_uvfits(tmp_path):
     testfile = str(tmp_path / "outtest_MWAcorr.uvfits")
     mwa_uv.write_uvfits(testfile, spoof_nonessential=True)
     uvfits_uv.read_uvfits(testfile)
+
+    # make sure filenames are what we expect
+    assert set(mwa_uv.filename) == {
+        "1131733552.metafits",
+        "1131733552_20151116182537_mini_gpubox01_00.fits",
+    }
+    assert uvfits_uv.filename == ["outtest_MWAcorr.uvfits"]
+    mwa_uv.filename = uvfits_uv.filename
+    mwa_uv._filename.form = (1,)
+
     assert mwa_uv == uvfits_uv
 
+
+@pytest.mark.filterwarnings("ignore:antnums_to_baseline")
+@pytest.mark.filterwarnings("ignore:Found antenna numbers > 256 in this data")
+def test_read_mwax_write_uvfits(tmp_path):
+    """
+    MWAX correlator fits to uvfits loopback test.
+
+    Read in MWAX correlator files, write out as uvfits, read back in and check
+    for object equality.
+    """
+    # spoof testfile to contain 2 times and 2 freqs
+    spoof_file = str(tmp_path / "mwax_spoof_ch137_000.fits")
+    with fits.open(filelist[12]) as mini1:
+        mini1[0].header["NFINECHS"] = 2
+        mini1[1].data = np.repeat(mini1[1].data, 2, axis=1)
+        mini1[2].data = np.repeat(mini1[2].data, 2, axis=1)
+        extra_dat = np.copy(mini1[1].data)
+        extra_samps = np.copy(mini1[2].data)
+        # mini1.append(fits.HDUList([mini1[1], mini1[2]]))
+        mini1.append(fits.ImageHDU(extra_dat))
+        mini1.append(fits.ImageHDU(extra_samps))
+        mini1[3].header["TIME"] = 1636374472
+        mini1[4].header["TIME"] = 1636374472
+        mini1[3].header["MILLITIM"] = 0
+        mini1[4].header["MILLITIM"] = 0
+        mini1.writeto(spoof_file)
+    mwax_uv = UVData()
+    uvfits_uv = UVData()
     messages = [
-        "telescope_location is not set",
         "some coarse channel files were not submitted",
     ]
     with uvtest.check_warnings(UserWarning, messages):
-        mwa_uv.read_mwa_corr_fits(
-            filelist[0:2], correct_cable_len=True, phase_to_pointing_center=True
+        mwax_uv.read_mwa_corr_fits(
+            [spoof_file, filelist[11]],
+            correct_cable_len=True,
+            phase_to_pointing_center=True,
         )
-
-    testfile = str(tmp_path / "outtest_MWAcorr.uvfits")
-    mwa_uv.write_uvfits(testfile, spoof_nonessential=True)
+    testfile = str(tmp_path / "outtest_MWAXcorr.uvfits")
+    mwax_uv.write_uvfits(testfile, spoof_nonessential=True)
     uvfits_uv.read_uvfits(testfile)
-    assert mwa_uv == uvfits_uv
+
+    assert mwax_uv == uvfits_uv
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
+def test_mwax_metafits_keys(tmp_path):
+    """Check that mwax keywords are removed from extra_keywords for legacy files"""
+    meta_spoof_file = str(tmp_path / "spoof_1131733552.metafits")
+    with fits.open(filelist[0]) as meta:
+        meta[0].header["DELAYMOD"] = None
+        meta.writeto(meta_spoof_file)
+    uv = UVData()
+    uv.read([meta_spoof_file, filelist[1]])
+
+    assert "DELAYMOD" not in uv.extra_keywords.keys()
+
+
+@pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_select_on_read():
     mwa_uv = UVData()
     mwa_uv2 = UVData()
@@ -119,7 +178,6 @@ def test_select_on_read():
         UserWarning,
         [
             'Warning: select on read keyword set, but file_type is "mwa_corr_fits"',
-            "telescope_location is not set. Using known values for MWA.",
             "some coarse channel files were not submitted",
         ],
     ):
@@ -131,7 +189,6 @@ def test_select_on_read():
     assert mwa_uv == mwa_uv2
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
 def test_read_mwa_read_cotter():
     """
@@ -148,6 +205,7 @@ def test_read_mwa_read_cotter():
         correct_cable_len=True,
         remove_dig_gains=False,
         remove_coarse_band=False,
+        remove_flagged_ants=False,
     )
     cotter_uv.read(filelist[6])
     # cotter doesn't record the auto xy polarizations
@@ -162,6 +220,7 @@ def test_read_mwa_read_cotter():
         atol=1e-4,
         rtol=0,
     )
+    assert mwa_uv.freq_array == cotter_uv.freq_array
 
 
 def test_read_mwa_write_uvfits_meta_mod(tmp_path):
@@ -176,7 +235,6 @@ def test_read_mwa_write_uvfits_meta_mod(tmp_path):
     mwa_uv = UVData()
     uvfits_uv = UVData()
     messages = [
-        "telescope_location is not set",
         "some coarse channel files were not submitted",
     ]
     files = [filelist[1], filelist[5]]
@@ -185,12 +243,22 @@ def test_read_mwa_write_uvfits_meta_mod(tmp_path):
     testfile = str(tmp_path / "outtest_MWAcorr.uvfits")
     mwa_uv.write_uvfits(testfile, spoof_nonessential=True)
     uvfits_uv.read_uvfits(testfile)
+
+    # make sure filenames are what we expect
+    assert set(mwa_uv.filename) == {
+        "1131733552_20151116182537_mini_gpubox01_00.fits",
+        "1131733552_mod.metafits",
+    }
+    assert uvfits_uv.filename == ["outtest_MWAcorr.uvfits"]
+    mwa_uv.filename = uvfits_uv.filename
+    mwa_uv._filename.form = (1,)
+
     assert mwa_uv == uvfits_uv
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
-@pytest.mark.filterwarnings("ignore:Combined frequencies are not contiguous")
+@pytest.mark.filterwarnings("ignore:Combined frequencies are separated by more than")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_read_mwa_multi():
     """Test reading in two sets of files."""
     set1 = filelist[0:2]
@@ -201,11 +269,11 @@ def test_read_mwa_multi():
 
     mwa_uv2 = UVData()
     messages = [
-        "telescope_location is not set",
         "some coarse channel files were not submitted",
-        "telescope_location is not set",
+        "cable length correction is now defaulted to True",
         "some coarse channel files were not submitted",
-        "Combined frequencies are not contiguous",
+        "cable length correction is now defaulted to True",
+        "Combined frequencies are separated by more than their channel width",
     ]
     with uvtest.check_warnings(UserWarning, messages):
         mwa_uv2.read([set1, set2], file_type="mwa_corr_fits")
@@ -213,8 +281,9 @@ def test_read_mwa_multi():
     assert mwa_uv == mwa_uv2
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:Combined frequencies are separated by more than")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_read_mwa_multi_concat(tmp_path):
     """Test reading in two sets of files with fast concatenation."""
     # modify file so that time arrays are matching
@@ -229,10 +298,11 @@ def test_read_mwa_multi_concat(tmp_path):
 
     mwa_uv2 = UVData()
     messages = [
-        "telescope_location is not set",
         "some coarse channel files were not submitted",
-        "telescope_location is not set",
+        "cable length correction is now defaulted to True",
         "some coarse channel files were not submitted",
+        "cable length correction is now defaulted to True",
+        "Combined frequencies are separated by more than their channel width",
     ]
     with uvtest.check_warnings(UserWarning, messages):
         mwa_uv2.read([set1, set2], axis="freq", file_type="mwa_corr_fits")
@@ -241,25 +311,24 @@ def test_read_mwa_multi_concat(tmp_path):
     os.remove(mod_mini_6)
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
 def test_read_mwa_flags():
     """Test handling of flag files."""
     mwa_uv = UVData()
     subfiles = [filelist[0], filelist[1], filelist[3], filelist[4]]
     messages = [
-        "mwaf files submitted with use_cotter_flags=False",
-        "telescope_location is not set",
+        "mwaf files submitted with use_aoflagger_flags=False",
         "some coarse channel files were not submitted",
+        "cable length correction is now defaulted to True",
     ]
     with uvtest.check_warnings(UserWarning, messages):
-        mwa_uv.read(subfiles, use_cotter_flags=False)
+        mwa_uv.read(subfiles, use_aoflagger_flags=False)
 
     del mwa_uv
 
     mwa_uv = UVData()
     with pytest.raises(ValueError) as cm:
-        mwa_uv.read(subfiles[0:2], use_cotter_flags=True)
+        mwa_uv.read(subfiles[0:2], use_aoflagger_flags=True)
     assert str(cm.value).startswith("no flag files submitted")
     del mwa_uv
 
@@ -271,14 +340,14 @@ def test_multiple_coarse():
     Read in MWA correlator files with two different orderings of the files
     and check for object equality.
     """
-    order1 = [filelist[0:3]]
+    order1 = filelist[0:3]
     order2 = [filelist[0], filelist[2], filelist[1]]
     mwa_uv1 = UVData()
     mwa_uv2 = UVData()
     messages = [
-        "telescope_location is not set",
         "coarse channels are not contiguous for this observation",
         "some coarse channel files were not submitted",
+        "cable length correction is now defaulted to True",
     ]
     with uvtest.check_warnings(UserWarning, messages):
         mwa_uv1.read(order1)
@@ -288,8 +357,8 @@ def test_multiple_coarse():
     assert mwa_uv1 == mwa_uv2
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_ppds(tmp_path):
     """Test handling of ppds files"""
     # turnaround test with just ppds file given
@@ -301,6 +370,16 @@ def test_ppds(tmp_path):
     mwa_uv.write_uvfits(testfile, spoof_nonessential=True)
     uvfits_uv = UVData()
     uvfits_uv.read_uvfits(testfile)
+
+    # make sure filenames are what we expect
+    assert set(mwa_uv.filename) == {
+        "1131733552_20151116182537_mini_gpubox01_00.fits",
+        "1131733552_metafits_ppds.fits",
+    }
+    assert uvfits_uv.filename == ["outtest_MWAcorr.uvfits"]
+    mwa_uv.filename = uvfits_uv.filename
+    mwa_uv._filename.form = (1,)
+
     assert mwa_uv == uvfits_uv
 
     del mwa_uv
@@ -326,10 +405,30 @@ def test_fine_channels(tmp_path):
         mini6.writeto(bad_fine)
     with pytest.raises(ValueError) as cm:
         mwa_uv.read([bad_fine, filelist[1]])
-    assert str(cm.value).startswith("files submitted have different fine")
+    assert str(cm.value).startswith("files submitted have different numbers")
     del mwa_uv
 
 
+def test_fine_channels_mwax(tmp_path):
+    """
+    Break read_mwa_corr_fits by submitting mwax files with different fine channels.
+
+    Test that error is raised if files with different numbers of fine channels
+    are submitted.
+    """
+    mwax_uv = UVData()
+    bad_fine = str(tmp_path / "bad_ch137_000.fits.fits")
+    with fits.open(filelist[12]) as mini:
+        mini[1].data = np.repeat(mini[1].data, 2, axis=1)
+        mini[2].data = np.repeat(mini[2].data, 2, axis=1)
+        mini[0].header["NFINECHS"] = 2
+        mini.writeto(bad_fine)
+    with pytest.raises(ValueError) as cm:
+        mwax_uv.read([bad_fine, filelist[12]])
+    assert str(cm.value).startswith("files submitted have different numbers")
+    del mwax_uv
+
+
 @pytest.mark.parametrize(
     "files,err_msg",
     [
@@ -403,29 +502,31 @@ def test_misaligned_times(tmp_path):
     del mwa_uv
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings(
     "ignore:coarse channels are not contiguous for this observation"
 )
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_flag_nsample_basic():
     """
     Test that the flag(without flag_int) and nsample arrays correctly reflect data.
     """
     uv = UVData()
-    uv.read_mwa_corr_fits(filelist[0:3], flag_init=False, propagate_coarse_flags=False)
+    uv.read_mwa_corr_fits(
+        filelist[0:3],
+        flag_init=False,
+        propagate_coarse_flags=False,
+        remove_flagged_ants=False,
+    )
     # check that only bad antennas are flagged for all times, freqs, pols
     bad_ants = [59, 114]
-    good_ants = list(range(128))
-    for j in bad_ants:
-        good_ants.remove(j)
-    bad = uv.select(antenna_nums=bad_ants, inplace=False)
+    good_ants = np.delete(np.unique(uv.ant_1_array), bad_ants)
+    bad = uv.select(antenna_nums=np.unique(uv.ant_1_array)[bad_ants], inplace=False)
     good = uv.select(antenna_nums=good_ants, inplace=False)
     assert np.all(bad.flag_array)
     good.flag_array = good.flag_array.reshape(
         (good.Ntimes, good.Nbls, good.Nfreqs, good.Npols)
     )
-    print(good.flag_array.shape)
     # good ants should be flagged except for the first time and second freq,
     # and for the second time and first freq
     assert np.all(good.flag_array[1:-1, :, :, :])
@@ -444,11 +545,11 @@ def test_flag_nsample_basic():
     assert np.all(uv.nsample_array[-1, :, 1, :] == 0.0)
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings(
     "ignore:coarse channels are not contiguous for this observation"
 )
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_flag_init(flag_file_init):
     """
     Test that routine MWA flagging works as intended.
@@ -473,11 +574,11 @@ def test_flag_init(flag_file_init):
     ), "Some non-edge/center channels are entirely flagged!"
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings(
     "ignore:coarse channels are not contiguous for this observation"
 )
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_flag_start_flag(flag_file_init):
     uv = UVData()
     uv.read(
@@ -522,11 +623,11 @@ def test_flag_start_flag(flag_file_init):
         ),
     ],
 )
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings(
     "ignore:coarse channels are not contiguous for this observation"
 )
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_flag_init_errors(flag_file_init, err_type, read_kwargs, err_msg):
     uv = UVData()
     # give noninteger multiple inputs
@@ -539,7 +640,6 @@ def test_read_metadata_only(tmp_path):
     """Test reading an MWA corr fits file as metadata only."""
     uvd = UVData()
     messages = [
-        "telescope_location is not set",
         "some coarse channel files were not submitted",
     ]
     with uvtest.check_warnings(UserWarning, messages):
@@ -553,8 +653,8 @@ def test_read_metadata_only(tmp_path):
     assert uvd.metadata_only
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_data_array_precision():
     uv = UVData()
     uv2 = UVData()
@@ -570,8 +670,8 @@ def test_data_array_precision():
     return
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_nsample_array_precision():
     uv = UVData()
     uv2 = UVData()
@@ -605,69 +705,86 @@ def test_invalid_precision_errors():
     return
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_remove_dig_gains():
     """Test digital gain removal."""
     uv1 = UVData()
-    uv1.read(filelist[0:2])
+    uv1.read(filelist[0:2], data_array_dtype=np.complex64)
 
     uv2 = UVData()
     uv2.read(filelist[0:2], remove_dig_gains=False)
 
     with fits.open(filelist[0]) as meta:
         meta_tbl = meta[1].data
-        antenna_numbers = meta_tbl["Antenna"][1::2]
+        antenna_inds = meta_tbl["Antenna"][1::2]
         dig_gains = meta_tbl["Gains"][1::2, :].astype(np.float64) / 64
-    reordered_inds = antenna_numbers.argsort()
-    dig_gains = dig_gains[reordered_inds, :]
-    dig_gains = dig_gains[:, np.array([23])]
-    dig_gains = np.repeat(dig_gains, 1, axis=1)
-    dig_gains1 = dig_gains[uv2.ant_1_array, :]
-    dig_gains2 = dig_gains[uv2.ant_2_array, :]
-    dig_gains1 = dig_gains1[:, :, np.newaxis, np.newaxis]
-    dig_gains2 = dig_gains2[:, :, np.newaxis, np.newaxis]
-    uv2.data_array = uv2.data_array / (dig_gains1 * dig_gains2)
+    reordered_inds = antenna_inds.argsort()
+    bad_ant_inds = [59, 114]
+    good_ants = np.delete(reordered_inds, bad_ant_inds)
+    ant_1_inds, ant_2_inds = np.transpose(
+        list(itertools.combinations_with_replacement(np.arange(uv2.Nants_data), 2))
+    )
+    ant_1_inds = np.tile(np.array(ant_1_inds), uv2.Ntimes).astype(np.int_)
+    ant_2_inds = np.tile(np.array(ant_2_inds), uv2.Ntimes).astype(np.int_)
+    dig_gains = dig_gains[good_ants, 23]
+    uv2.data_array = uv2.data_array / (
+        dig_gains[ant_1_inds, np.newaxis, np.newaxis, np.newaxis]
+        * dig_gains[ant_2_inds, np.newaxis, np.newaxis, np.newaxis]
+    )
+    uv2.history = uv1.history
 
+    # make sure correction doesn't change data_array type
+    assert uv1.data_array.dtype == np.complex64
+    assert "Divided out digital gains" in uv1.history
     assert uv1 == uv2
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
-def test_remove_coarse_band():
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
+def test_remove_coarse_band(tmp_path):
     """Test coarse band removal."""
+    # generate a spoof file with 32 channels
+    cb_spoof = str(tmp_path / "cb_spoof_01_00.fits")
+    with fits.open(filelist[1]) as mini1:
+        mini1[1].data = np.repeat(mini1[1].data, 32, axis=0)
+        mini1.writeto(cb_spoof)
+
     uv1 = UVData()
-    uv1.read(filelist[0:2])
+    uv1.read([filelist[0], cb_spoof], data_array_dtype=np.complex64)
 
     uv2 = UVData()
-    uv2.read(filelist[0:2], remove_coarse_band=False)
+    uv2.read([filelist[0], cb_spoof], remove_coarse_band=False)
 
-    with open(DATA_PATH + "/mwa_config_data/MWA_rev_cb_10khz_doubles.txt", "r") as f:
-        cb = f.read().splitlines()
-    cb_array = np.array(cb).astype(np.float64)
-    cb_array = cb_array.reshape(32, 4)
+    with h5py.File(
+        DATA_PATH + "/mwa_config_data/MWA_rev_cb_10khz_doubles.h5", "r"
+    ) as f:
+        cb = f["coarse_band"][:]
+    cb_array = cb.reshape(32, 4)
     cb_array = np.average(cb_array, axis=1)
-    cb_array = cb_array[0]
+    uv2.data_array /= cb_array[:, np.newaxis]
 
-    uv2.data_array = uv2.data_array.swapaxes(2, 3)
-    uv2.data_array = uv2.data_array / cb_array
-    uv2.data_array = uv2.data_array.swapaxes(2, 3)
+    uv2.history = uv1.history
 
+    # make sure correction doesn't change data_array type
+    assert uv1.data_array.dtype == np.complex64
+    assert "Divided out pfb coarse channel bandpass" in uv1.history
     assert uv1 == uv2
 
 
-def test_cotter_flags():
-    """Test using cotter flags"""
+def test_aoflagger_flags():
+    """Test using aoflagger flags"""
     uv = UVData()
     files = filelist[0:2]
     files.append(filelist[3])
     messages = [
-        "telescope_location is not set.",
         "some coarse channel files were not submitted",
         "coarse channel, start time, and end time flagging will default",
     ]
     with uvtest.check_warnings(UserWarning, messages):
-        uv.read_mwa_corr_fits(files, flag_init=False)
+        uv.read_mwa_corr_fits(
+            files, flag_init=False, remove_flagged_ants=False, correct_cable_len=False,
+        )
 
     with fits.open(filelist[3]) as aoflags:
         flags = aoflags[1].data.field("FLAGS")
@@ -677,14 +794,14 @@ def test_cotter_flags():
     assert np.all(uv.flag_array == flags)
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
 @pytest.mark.filterwarnings(
     "ignore:coarse channels are not contiguous for this observation"
 )
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
 @pytest.mark.filterwarnings("ignore:coarse channel, start time, and end time flagging")
-def test_cotter_flags_multiple(tmp_path):
-    """Test cotter flags with multiple coarse bands"""
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
+def test_aoflagger_flags_multiple(tmp_path):
+    """Test aoflagger flags with multiple coarse bands"""
     mod_mini_6 = str(tmp_path / "mini_gpubox06_01.fits")
     with fits.open(filelist[2]) as mini6:
         mini6[1].header["time"] = 1447698337
@@ -693,7 +810,7 @@ def test_cotter_flags_multiple(tmp_path):
     files.append(mod_mini_6)
 
     uv = UVData()
-    uv.read_mwa_corr_fits(files, flag_init=False)
+    uv.read_mwa_corr_fits(files, flag_init=False, remove_flagged_ants=False)
 
     with fits.open(filelist[3]) as aoflags:
         flags1 = aoflags[1].data.field("FLAGS")
@@ -707,9 +824,9 @@ def test_cotter_flags_multiple(tmp_path):
     assert np.all(uv.flag_array == flags)
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
 @pytest.mark.filterwarnings("ignore:coarse channel, start time, and end time flagging")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_mismatch_flags():
     """Break by submitting flag and gpubox files from different coarse bands."""
     uv = UVData()
@@ -720,11 +837,11 @@ def test_mismatch_flags():
     assert str(cm.value).startswith("flag file coarse bands do not match")
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings(
     "ignore:coarse channels are not contiguous for this observation"
 )
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_propagate_coarse_flags():
     """
     Test that the flag(without flag_int) and nsample arrays correctly reflect data.
@@ -734,8 +851,8 @@ def test_propagate_coarse_flags():
     assert np.all(uv.flag_array)
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_start_flag(tmp_path):
     """Test the default value of start_flag."""
     uv1 = UVData()
@@ -747,8 +864,7 @@ def test_start_flag(tmp_path):
         edge_width=0,
         flag_dc_offset=False,
     )
-    good_ants = np.arange(128)
-    good_ants = np.delete(good_ants, [59, 114])
+    good_ants = np.delete(np.unique(uv1.ant_1_array), [59, 114])
     uv1.select(antenna_nums=good_ants)
     # start_time is after goodtime, so data for good antennas should be unflagged
     assert np.all(~uv1.flag_array)
@@ -769,8 +885,8 @@ def test_start_flag(tmp_path):
     assert np.all(uv2.flag_array)
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_start_flag_goodtime_ppds():
     """Test that error is thrown using 'goodtime' with only ppds file."""
     uv = UVData()
@@ -779,8 +895,8 @@ def test_start_flag_goodtime_ppds():
     assert str(cm.value).startswith("To use start_flag='goodtime',")
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_start_flag_bad_string():
     """Test that error is thrown if start_flag is given string other than 'goodtime'"""
     uv = UVData()
@@ -789,6 +905,23 @@ def test_start_flag_bad_string():
     assert str(cm.value).startswith("start_flag must be int or float or 'goodtime'")
 
 
+@pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
+def test_start_flag_int_time(tmp_path):
+    """Test goodtime returning a start_flag smaller than integration time."""
+    uv = UVData()
+    new_meta = str(tmp_path / "1131733552_goodtime.metafits")
+    with fits.open(filelist[0]) as meta:
+        meta[0].header["GOODTIME"] = 1447698337.25
+        meta.writeto(new_meta)
+    uv.read(
+        [new_meta, filelist[1]], flag_init=True, start_flag="goodtime",
+    )
+    # first integration time should be flagged
+    # data only has one integration time, so all data should be flagged
+    assert np.all(uv.flag_array)
+
+
 def test_input_output_mapping():
     """Test the input_output_mapping function."""
     mapping_dict = {}
@@ -809,3 +942,138 @@ def test_input_output_mapping():
     assert function_output == mapping_dict
 
     return
+
+
+@pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+def test_van_vleck_int():
+    """Test van vleck correction integral implementation."""
+    uv1 = UVData()
+    uv1.read(
+        filelist[8:10],
+        flag_init=False,
+        data_array_dtype=np.complex64,
+        correct_van_vleck=True,
+        cheby_approx=False,
+        remove_coarse_band=False,
+        remove_dig_gains=False,
+        remove_flagged_ants=False,
+        correct_cable_len=False,
+    )
+    # read in file corrected using integrate.quad with 1e-10 precision
+    uv2 = UVData()
+    uv2.read(filelist[10])
+    assert np.allclose(uv1.data_array, uv2.data_array)
+
+
+@pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+def test_van_vleck_cheby():
+    """Test van vleck correction chebyshev implementation."""
+    uv1 = UVData()
+    uv1.read(
+        filelist[8:10],
+        flag_init=False,
+        correct_van_vleck=True,
+        cheby_approx=True,
+        remove_coarse_band=False,
+        remove_dig_gains=False,
+        correct_cable_len=False,
+    )
+    # read in file corrected using integrate.quad with 1e-10 precision
+    uv2 = UVData()
+    uv2.read(filelist[10])
+
+    # select only good ants
+    good_ants = np.delete(np.unique(uv2.ant_1_array), 76)
+    uv2.select(antenna_nums=good_ants)
+
+    assert np.allclose(uv1.data_array, uv2.data_array)
+
+
+def test_van_vleck_interp(tmp_path):
+    """Test van vleck correction with sigmas out of cheby interpolation range."""
+    small_sigs = str(tmp_path / "small_sigs07_01.fits")
+    with fits.open(filelist[8]) as mini:
+        mini[1].data = np.full((1, 66048), 7744)
+        mini.writeto(small_sigs)
+    messages = [
+        "values are being corrected with the van vleck integral",
+    ]
+    messages = messages * 10
+    messages.append("some coarse channel files were not submitted")
+    uv = UVData()
+    with uvtest.check_warnings(UserWarning, messages):
+        uv.read(
+            [small_sigs, filelist[9]],
+            flag_init=False,
+            correct_van_vleck=True,
+            cheby_approx=True,
+            remove_coarse_band=False,
+            remove_dig_gains=False,
+            correct_cable_len=False,
+        )
+
+
+@pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
+def test_remove_flagged_ants(tmp_path):
+    """Test remove_flagged_ants."""
+    uv1 = UVData()
+    uv1.read(
+        filelist[8:10], remove_flagged_ants=True,
+    )
+    uv2 = UVData()
+    uv2.read(
+        filelist[8:10], remove_flagged_ants=False,
+    )
+    good_ants = np.delete(np.unique(uv2.ant_1_array), 76)
+
+    uv2.select(antenna_nums=good_ants)
+
+    assert uv1 == uv2
+
+
+@pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:.*values are being corrected with the van vleck")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
+def test_small_sigs(tmp_path):
+    """Test flag_small_auto_ants."""
+    small_sigs = str(tmp_path / "small_sigs07_02.fits")
+    with fits.open(filelist[8]) as mini:
+        mini[1].data[0, 0] = 1000
+        mini.writeto(small_sigs)
+    uv1 = UVData()
+    uv1.read(
+        [small_sigs, filelist[9]], correct_van_vleck=True, flag_small_auto_ants=True,
+    )
+    messages = [
+        "values are being corrected with the van vleck integral",
+    ]
+    messages = messages * 8
+    messages.append("some coarse channel files were not submitted")
+    messages.append("cable length correction is now defaulted to True")
+    uv2 = UVData()
+    with uvtest.check_warnings(UserWarning, messages):
+        uv2.read(
+            [small_sigs, filelist[9]],
+            correct_van_vleck=True,
+            flag_small_auto_ants=False,
+        )
+
+    assert "flagged by the Van Vleck" in uv1.history
+    assert uv2.Nants_data - uv1.Nants_data == 1
+
+
+@pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+def test_deprecated_keywords():
+    """Test warnings for deprecated keywords"""
+    uv = UVData()
+    messages = ["Use `use_aoflagger_flags`"]
+    messages.append("Use `flag_small_auto_ants`")
+    messages.append("some coarse channel files were not submitted")
+    messages.append("cable length correction is now defaulted to True")
+    with uvtest.check_warnings(
+        [DeprecationWarning, DeprecationWarning, UserWarning, UserWarning], messages,
+    ):
+        uv.read(
+            filelist[0:2], use_cotter_flags=False, flag_small_sig_ants=True,
+        )
